/**
* @author Dirk Nordmann
* @date   14.01.2002
*/
package img2wmf;

import java.awt.Image;
import java.awt.image.ImageObserver;
import java.awt.image.PixelGrabber;
import java.io.*;

public class ImageToBMP
{
   	private Image _image;
   	private int _width;
   	private int _height;
   	private int[] _pixelMap;

   	
   	
    /* ######################################################################
       ###  CONSTRUCTOR
       ###
       ###  IN:  image			Image-Object to be converted to BMP
       ###  OUT: ImageToBmp		New instance of ImageToBmp
       ######################################################################
     */
    public ImageToBMP(Image image)
    {
   		_image = image;
   		do
   		{
       		_width = _image.getWidth(null);
       		_height = _image.getHeight(null);
   		}
   		while 
   				(_width < 0 || _height < 0);
   		_pixelMap = new int[_width * _height];
    }

    
    
  public boolean convert(DataOutputStream dataoutputstream)
    throws IOException
  {
    boolean flag = getPixelMap();
   	if (flag)
		{
			flag = convert8Bit(dataoutputstream);
			if (!flag)
				flag = convert24Bit(dataoutputstream);
		}	
		return flag;
  }
    
    
    
    private boolean convert24Bit(DataOutputStream dataoutputstream)
        throws IOException
    {
        //*****  Write headers to OutputStream  *****
        
        int k = ((_width * 3) / 4) * 4;
        if(k != _width * 3)
            k += 4;
        byte byte0 = 54;
        int l = k * _height;
        writeHeader(dataoutputstream, byte0, l);
        writeBMPInfoHeader(dataoutputstream, _width, _height, 24);

        
        //*****  Write pixels to OutputStream  *****
        
        int offs = _width * _height;
        for(int y = _height; y > 0; y--)
        {
            offs -= _width;
            for(int x = 0; x < _width; x++)
            {
            	// Get pixel
                int pixelColor = _pixelMap[x + offs];
                
                // Write pixel-color
                if((pixelColor & 0xff000000) == 0) {
                  // Background-Color: Color.white
                  dataoutputstream.writeByte(0xff);
                  dataoutputstream.writeByte(0xff);
                  dataoutputstream.writeByte(0xff);
                } else {
                  dataoutputstream.writeByte(pixelColor & 0xff);
                  dataoutputstream.writeByte((pixelColor & 0xff00) >> 8);
                  dataoutputstream.writeByte((pixelColor & 0xff0000) >> 16);
                }
            }

            int i2 = k - _width * 3;
            for(int j2 = 0; j2 < i2; j2++)
                dataoutputstream.writeByte(0);
        }

        return true;
    }

    
    
    private boolean convert8Bit(DataOutputStream dataoutputstream)
        throws IOException
    {
        //*****  Build color map  *****
        
        int colorMap[] = new int[256];
        int colorCount = 0;
        for(int pixelIndex = 0; pixelIndex < _width * _height && colorCount < colorMap.length; pixelIndex++)
        {
            boolean foundInColorMap = false;
            for(int colorIndex = 0; colorIndex < colorCount && !foundInColorMap; colorIndex++)
                if(colorMap[colorIndex] == _pixelMap[pixelIndex])
                    foundInColorMap = true;

            if (!foundInColorMap)
            {
                colorMap[colorCount] = _pixelMap[pixelIndex];
                colorCount++;
            }
            if(colorCount >= colorMap.length)
                return false;
        }

        
        //*****  Write headers to OutputStream  *****
        
        char c = '\u0436';
        int j1 = (_width / 4) * 4;
        if(j1 != _width)
            j1 += 4;
        int k1 = j1 * _height;
        writeHeader(dataoutputstream, c, k1);
        writeBMPInfoHeader(dataoutputstream, _width, _height, 8);

        
        //*****  Write color map to OutputStream  *****
        
        // Write found colors from color map to OutputStream
        for(int colorIndex = 0; colorIndex < colorCount; colorIndex++)
            if((colorMap[colorIndex] & 0xff000000) == 0) {
              // Background-Color: Color.white
              dataoutputstream.writeByte(0xff);
              dataoutputstream.writeByte(0xff);
              dataoutputstream.writeByte(0xff);
              dataoutputstream.writeByte(0);
            } else
                dataoutputstream.writeInt(swap(colorMap[colorIndex] & 0xffffff));
            
		// Write empty color map entries to OutputStream
        for(int i2 = colorCount; i2 < 256; i2++)
            dataoutputstream.writeInt(0);

            
        //*****  Write pixels to OutputStream  *****
        
        int offs = _width * _height;
        for(int y = _height; y > 0; y--)
        {
            offs -= _width;
            for(int x = 0; x < _width; x++)
            {
            	// Get pixel
                int pixelColor = _pixelMap[x + offs];
                
                // Write entry for pixel-color in colorMap
                for(int colorIndex = 0; colorIndex < colorCount; colorIndex++)
                    if (colorMap[colorIndex] == pixelColor)
                    {
		                    dataoutputstream.writeByte(colorIndex);
    		                break;
                    }
            }

            int i2 = j1 - _width;
            for(int x2 = 0; x2 < i2; x2++)
                dataoutputstream.writeByte(0);

        }

        return true;
    }

    
    
    private int swap(int i)
    {
        int j = (i & 0xff000000) >> 24;
        j += (i & 0xff0000) >> 8;
        j += (i & 0xff00) << 8;
        j += (i & 0xff) << 24;
        return j;
    }

    private int swap(short word0)
    {
        int i = (word0 & 0xff00) >> 8;
        i += (word0 & 0xff) << 8;
        return (short)i;
    }

    private boolean getPixelMap()
    {
        PixelGrabber pixelgrabber = new PixelGrabber(_image, 0, 0, _width, _height, _pixelMap, 0, _width);
        try
        {
            pixelgrabber.grabPixels();
        }
        catch(InterruptedException _ex)
        {
            return false;
        }
        if((pixelgrabber.status() & 0x80) != 0)
        {
            System.err.println("[ImageToBmp]: ***** ERROR: Cannot get image pixel data.");
            return false;
        } else
        {
            return true;
        }
    }

    private void writeHeader(DataOutputStream dataoutputstream, int i, int j)
        throws IOException
    {
        dataoutputstream.writeByte(66);
        dataoutputstream.writeByte(77);
        dataoutputstream.writeInt(swap(i + j));
        dataoutputstream.writeInt(0);
        dataoutputstream.writeInt(swap(i));
    }

    private void writeBMPInfoHeader(DataOutputStream dataoutputstream, int i, int j, int k)
        throws IOException
    {
        dataoutputstream.writeInt(swap(40));
        dataoutputstream.writeInt(swap(i));
        dataoutputstream.writeInt(swap(j));
        dataoutputstream.writeShort(swap((short)1));
        dataoutputstream.writeShort(swap((short)k));
        dataoutputstream.writeInt(0);
        dataoutputstream.writeInt(0);
        dataoutputstream.writeInt(0);
        dataoutputstream.writeInt(0);
        dataoutputstream.writeInt(0);
        dataoutputstream.writeInt(0);
    }

}