/**
* @author Dirk Nordmann
* @date   14.01.2002
*/
package xml2rtf.rtf;

import util.*;

public class RtfHeading 
{
  private int level;
  
  /**
  * Constructor
  */
  public RtfHeading(int level)
  {
    this.setLevel(level);
  }

  /**
  * Sets the level for this heading-tag.
  * @param newLevel
  */
  public void setLevel(int newLevel) 
  {
    this.level = newLevel;
  }
  
  /**
  * Returns the level of this heading-tag.
  * @return int
  */
  public int getLevel() 
  {
    return this.level;
  }
  
  /**
  * Processes RTF representation of heading.
  * @param   rtfgenerator	RTF-Generator Object.
  */
  public void processPre(RtfGenerator rtfgenerator)
  {
  	rtfgenerator.outputRtf
    	( "\n\\pard\\plain" 
      + "\n\\s" + this.getLevel() + "\\ql \\li0\\ri0"
  		+ H_TITLE_BEFORE[this.getLevel()-1] 
    	+ "\n{"
    );
  }

  /**
  * Processes RTF representation of heading.
  * @param   rtfgenerator	RTF-Generator Object.
  */
  public void processPost(RtfGenerator rtfgenerator)
  {
    rtfgenerator.outputRtf("}");
  }

  private static final String[] H_TITLE_BEFORE =
    { "\\pagebb\\sb360\\sl300\\slmult1\\keepn\\widctlpar\\aspalpha\\aspnum\\faauto\\outlinelevel0\\adjustright\\rin0\\lin0\\itap0 "
    + "\\b\\f1\\fs34\\cf" + RtfWriter.getColor("darkblue") + "\\lang1031\\langfe1031\\kerning32\\cgrid\\langnp1031\\langfenp1031 "
    , "\\sb240\\sa60\\sl300\\slmult1\\keepn\\widctlpar\\aspalpha\\aspnum\\faauto\\outlinelevel1\\adjustright\\rin0\\lin0\\itap0 "
    + "\\b\\i\\f1\\fs28\\lang1031\\langfe1031\\cgrid\\langnp1031\\langfenp1031"
    , "\\sb240\\sa60\\sl300\\slmult1\\keepn\\widctlpar\\aspalpha\\aspnum\\faauto\\outlinelevel2\\adjustright\\rin0\\lin0\\itap0 "
    + "\\b\\f1\\fs26\\cf" + RtfWriter.getColor("darkblue") + "\\lang1031\\langfe1031\\cgrid\\langnp1031\\langfenp1031"
    , "\\sb240\\sa60\\sl300\\slmult1\\keepn\\widctlpar\\aspalpha\\aspnum\\faauto\\outlinelevel3\\adjustright\\rin0\\lin0\\itap0 "
    + "\\b\\f1\\fs22\\lang1031\\langfe1031\\cgrid\\langnp1031\\langfenp1031"
    , "\\sl300\\slmult1\\widctlpar\\aspalpha\\aspnum\\faauto\\outlinelevel4\\adjustright\\rin0\\lin0\\itap0 "
    + "\\f1\\fs22\\lang1031\\langfe1031\\cgrid\\langnp1031\\langfenp1031"
    , "\\sl300\\slmult1\\widctlpar\\aspalpha\\aspnum\\faauto\\outlinelevel5\\adjustright\\rin0\\lin0\\itap0 "
    + "\\f1\\fs22\\lang1031\\langfe1031\\cgrid\\langnp1031\\langfenp1031"
    };
    
}