###################################################################################################
# _linklib.py
#
# Implementation of Link-Library (see below).
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
import AccessControl.Role
import string

# ---------------------------------------------------------------------------------------------
# getUrl
# ---------------------------------------------------------------------------------------------
def getUrl(self, key, REQUEST):
    
  #-- [ReqBuff]: Fetch buffered value from Http-Request.
  try:
    reqBuffId = 'getUrl_%s'%key
    return self.fetchReqBuff(reqBuffId,REQUEST)
  except:
  
    #-- Get value.
    url = self.getObjProperty(key,REQUEST)
    if isInternalLink(url):
      ob = getLinkObj(self,url,REQUEST)
      if ob is not None:
        internal_url = getInternalLink(ob.relative_obj_path()) 
        # Repair internal link.
        if url != internal_url:
          self.setObjProperty(key,internal_url,REQUEST['lang'])
    value = self.getLinkUrl(self.getObjProperty(key,REQUEST),REQUEST)
    
    #-- [ReqBuff]: Returns value and stores it in buffer of Http-Request.
    return self.storeReqBuff(reqBuffId,value,REQUEST)
    

###############################################################################################
# getInternalLink:
###############################################################################################
def getInternalLink(relative_obj_path):
  return '{$%s}'%relative_obj_path

###############################################################################################
# isInternalLink:
###############################################################################################
def isInternalLink(url):
  return url is not None and string.find(url,'{$')>=0 and string.find(url,'}')==len(url)-1

###############################################################################################
# isMailLink:
###############################################################################################
def isMailLink(url):
  return string.find(string.lower(url),'mailto:') >= 0

###############################################################################################
# getLinkObj:
#
# Resolves internal/external links and returns Object.
###############################################################################################
def getLinkObj(self, url, REQUEST):
  #++ print "[%s.getLinkObj]: %s"%(self.meta_type,url)
  ob = None
  if isInternalLink(url):
    path = url[2:-1]
    ob = self.getDocumentElement().findObjId(path,REQUEST)
  return ob

###############################################################################################
# getLinkUrl:
#
# Resolves internal/external links and returns URL.
###############################################################################################
def getLinkUrl(self, url, REQUEST):
  if isInternalLink(url):
    ob = getLinkObj(self,url,REQUEST)
    url = None
    if ob is not None:
      if ob.meta_type in ['ZMSFile']:
        url = ob.getFile(REQUEST).getHref(REQUEST)
      else:
        url = ob.getHref2IndexHtml(REQUEST)
  return url

###############################################################################################
# LinkDetails:
###############################################################################################
class LinkDetails(AccessControl.Role.RoleManager):

  # Constructor
  def __init__(self, ob):
    self.setUrl(ob['url'])
    self.setTitle(ob['title'])
    self.setDescription(ob['description'])
    
  def setUrl(self, url):
    self._url = url

  getUrl__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
  def getUrl(self):
    return self._url
  def getLinkObj(self, owner, REQUEST):
    obj = getLinkObj(owner,self.getUrl(),REQUEST)
    return obj
  def getDisplayUrl(self, owner, REQUEST):
    url = getLinkUrl(owner,self.getUrl(),REQUEST)
    title = string.lower(self.getDisplayTitle(owner,REQUEST))
    if title == 'medline':
      try:
        url = 'http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?cmd=Retrieve&db=PubMed&list_uids=%i&dopt=Abstract'%int(url)
      except:
        pass
    return url

  def setTitle(self, title):
    self._title = title

  getTitle__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
  def getTitle(self):
    return self._title
  def getDisplayTitle(self, owner, REQUEST):
    rtn = self.getTitle()
    if len(rtn) == 0:
      rtn = self.getUrl()
    return rtn

  def setDescription(self, description):
    self._description = description

  getDescription__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
  def getDescription(self):
    return self._description
  def getDisplayDescription(self, owner, REQUEST):
    rtn = ''
    s_description = self.getDescription()
    for s in string.split(s_description,'\n'):
      if len(s) > 0:
        if len(rtn) > 0:
          rtn = rtn + ' <br>'
        rtn = rtn + s
    return rtn

###################################################################################################
